###################################################################################
#
#    Copyright (C) 2020 Cetmix OÜ
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU LESSER GENERAL PUBLIC LICENSE as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU LESSER GENERAL PUBLIC LICENSE for more details.
#
#    You should have received a copy of the GNU LESSER GENERAL PUBLIC LICENSE
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###################################################################################

from odoo import api, models


class MailThread(models.AbstractModel):
    _inherit = "mail.thread"

    @api.model
    def message_new(self, msg_dict, custom_values=None):
        return super(MailThread, self.with_context(skip_notification=True)).message_new(
            msg_dict, custom_values=custom_values
        )

    def _message_notification_recipients(self, message, recipients):
        if self.env["ir.config_parameter"].sudo().get_param(
            "cetmix.mail_incoming_smart_notify", False
        ):
            # Filtering notification recipients
            # by message recipients from input mail message
            if self._context.get("skip_notification", False):
                recipients = self.env["res.partner"]
            else:
                mail_recipients = self._context.get("message_recipients", False)
                if mail_recipients:
                    recipients = recipients.filtered(
                        lambda partner: partner.id not in mail_recipients
                    )
        return super(MailThread, self)._message_notification_recipients(
            message, recipients
        )

    @api.model
    def message_route_process(self, message, message_dict, routes):
        # Get partner ids from recipients
        partner_ids = list(
            map(
                lambda rec: rec[1], message_dict.get("partner_ids", [])
            )
        )
        return super(MailThread, self.with_context(
            message_recipients=partner_ids
        )).message_route_process(
            message, message_dict, routes
        )

    def unlink(self):
        self.env["mail.message"].sudo().with_context(active_test=False).search([
            ("model", "=", self._name), ("res_id", "in", self.ids)
        ]).unlink()
        return super(MailThread, self).unlink()
