###################################################################################
#
#    Copyright (C) 2020 Cetmix OÜ
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU LESSER GENERAL PUBLIC LICENSE as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU LESSER GENERAL PUBLIC LICENSE for more details.
#
#    You should have received a copy of the GNU LESSER GENERAL PUBLIC LICENSE
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###################################################################################

from odoo import _, api, fields, models

from .common import DEFAULT_SIGNATURE_LOCATION


###################
# Config Settings #
###################
class ResConfigSettings(models.TransientModel):
    _inherit = "res.config.settings"

    messages_easy_text_preview = fields.Integer(
        string="Text preview length",
        config_parameter="cetmix.messages_easy_text_preview",
        default=200,
    )
    messages_easy_color_note = fields.Char(
        string="Note Background",
        help="Background color for internal notes in HTML format (e.g. #fbd78b)",
        config_parameter="cetmix.messages_easy_color_note",
    )
    messages_easy_empty_trash = fields.Integer(
        string="Empty trash in (days)",
        config_parameter="cetmix.messages_easy_empty_trash",
        default=0,
    )
    mail_incoming_smart_notify = fields.Boolean(
        string="Smart Notification",
        help="Do not notify followers "
             "if they are already notified "
             "in the incoming email",
        config_parameter="cetmix.mail_incoming_smart_notify",
        default=False,
    )
    message_signature_location = fields.Selection(
        selection=[
            ("a", "Message bottom"),
            ("b", "Before quote"),
            ("n", "No signature"),
        ],
        default=DEFAULT_SIGNATURE_LOCATION,
        string="Default Signature Location",
        help="Set default signature location",
        config_parameter="cetmix.message_signature_location",
    )

    # -- Save values
    def set_values(self):
        super(ResConfigSettings, self).set_values()
        ICPSudo = self.env["ir.config_parameter"].sudo()
        ICPSudo.set_param(
            "cetmix.messages_easy_text_preview", self.messages_easy_text_preview
        )
        ICPSudo.set_param(
            "cetmix.messages_easy_color_note", self.messages_easy_color_note
        )
        ICPSudo.set_param(
            "cetmix.messages_easy_empty_trash", self.messages_easy_empty_trash
        )
        ICPSudo.set_param(
            "cetmix.mail_incoming_smart_notify", self.mail_incoming_smart_notify
        )
        ICPSudo.set_param(
            "cetmix.message_signature_location", self.message_signature_location
        )

    # -- Read values
    @api.model
    def get_values(self):
        res = super(ResConfigSettings, self).get_values()
        ICPSudo = self.env["ir.config_parameter"].sudo()

        message_signature_location = ICPSudo.get_param(
            "cetmix.message_signature_location", default=DEFAULT_SIGNATURE_LOCATION
        )
        res.update(message_signature_location=message_signature_location)

        # Text preview length
        messages_easy_text_preview = ICPSudo.get_param(
            "cetmix.messages_easy_text_preview", default=False
        )
        if messages_easy_text_preview:
            res.update(messages_easy_text_preview=int(messages_easy_text_preview))

        # Internal note background color
        messages_easy_color_note = ICPSudo.get_param(
            "cetmix.messages_easy_color_note", default=False
        )
        if messages_easy_color_note:
            res.update(messages_easy_color_note=messages_easy_color_note)

        messages_easy_empty_trash = ICPSudo.get_param(
            "cetmix.messages_easy_empty_trash", default=False
        )
        if messages_easy_empty_trash:
            res.update(messages_easy_empty_trash=int(messages_easy_empty_trash))
        mail_incoming_smart_notify = ICPSudo.get_param(
            "cetmix.mail_incoming_smart_notify", default=False
        )
        res.update(mail_incoming_smart_notify=bool(mail_incoming_smart_notify))
        return res

    def action_configure_cron(self):
        return {
            "name": _("Edit cron"),
            "views": [(False, "form")],
            "res_model": "ir.cron",
            "res_id":
                self.env
                .ref("prt_mail_messages.ir_cron_ptr_mail_messages_action_unlink").id,
            "type": "ir.actions.act_window",
            "target": "new",
        }
