###################################################################################
#
#    Copyright (C) 2020 Cetmix OÜ
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU LESSER GENERAL PUBLIC LICENSE as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU LESSER GENERAL PUBLIC LICENSE for more details.
#
#    You should have received a copy of the GNU LESSER GENERAL PUBLIC LICENSE
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###################################################################################

from odoo.fields import first
from odoo.tests import SavepointCase
from odoo.tools import mute_logger


class TestSignatureLocation(SavepointCase):
    @classmethod
    def setUpClass(cls):
        super(TestSignatureLocation, cls).setUpClass()
        # Create Partner Kate
        cls.kate = cls.env["res.partner"].create(
            {"name": "Kate", "email": "kate@example.com"}
        )
        # Mail Message Test #1
        cls.mail_message_test_1 = cls.env["mail.message"].create(
            {"model": "res.partner", "res_id": cls.kate.id, "body": "Test Message #1"}
        )

        # Monkey patch to keep sent mails for further check
        def unlink_replacement(self):
            return

        cls.env["mail.mail"]._patch_method("unlink", unlink_replacement)

    def get_message(self, location):
        self.env["ir.config_parameter"].sudo().set_param(
            "cetmix.message_signature_location", location
        )
        composer = self.env["mail.compose.message"].with_context(
            **self.mail_message_test_1.with_context(
                wizard_mode="quote"
            ).reply_prep_context(),
        ).create({
            "partner_ids": [(6, 0, self.kate.ids)]
        })
        composer.send_mail()
        return first(self.kate.message_ids)

    @mute_logger("odoo.tests", "odoo.addons.mail.models.mail_mail")
    def test_mail_composer_signature_bottom_or_default(self):
        """Test signature location bottom/default"""
        message = self.get_message("a")
        signatire = message.create_uid.signature
        mail_ids = self.env["mail.mail"].search([("mail_message_id", "=", message.id)])
        body = mail_ids.body_html
        start_quote = body.find("<blockquote")
        self.assertNotIn(
            signatire,
            body[:start_quote],
            msg="Signature must not be before block <blockquote",
        )
        end_quote = body.find("</blockquote>")
        self.assertIn(
            signatire,
            body[end_quote:],
            msg="Signature must be after block </blockquote>",
        )

    @mute_logger("odoo.tests", "odoo.addons.mail.models.mail_mail")
    def test_mail_composer_signature_before(self):
        """Test signature location before quote"""
        message = self.get_message("b")
        signature = message.create_uid.signature
        mail_ids = self.env["mail.mail"].search([("mail_message_id", "=", message.id)])
        body = mail_ids.body_html
        start_quote = body.find("<blockquote")
        self.assertIn(
            signature,
            body[:start_quote],
            msg="Signature must be before block <blockquote>",
        )
        end_quote = body.find("</blockquote>")
        self.assertNotIn(
            signature,
            body[end_quote:],
            msg="Signature must not be after block </blockquote>",
        )

    @mute_logger("odoo.tests", "odoo.addons.mail.models.mail_mail")
    def test_mail_composer_no_signature(self):
        """Test without signature location"""
        message = self.get_message("n")
        signature = message.create_uid.signature
        mail_ids = self.env["mail.mail"].search([("mail_message_id", "=", message.id)])
        body = mail_ids.body_html
        self.assertNotIn(
            signature, body, msg="Signature must not contains in message body"
        )
